{% load i18n %}

<div class="social-auth-links text-center mb-3">
  <p>- {% translate "OR" %} -</p>
  <a class="btn btn-block btn-primary" href="#">
    <i class="fab fa-facebook mr-2"></i> {% translate "Sign in using Facebook" %}
  </a>
  <a class="btn btn-block btn-danger" href="#">
    <i class="fab fa-google-plus mr-2"></i> {% translate "Sign in using Google+" %}
  </a>
</div>