{% extends 'dj_accounts/auth_base.html' %}
{% load static i18n %}


{% block styles %}
<link href="https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.8/css/intlTelInput.css" rel="stylesheet">
<script src="https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.8/js/intlTelInput.min.js"></script>
{% endblock %}

{% block page_title %}
{% translate "Login" %}
{% endblock %}

{% block content %}
<div class="card">
  <div class="card-body login-card-body">
    <p class="login-box-msg">{% translate 'Log in to start your session' %}</p>

    {% include 'dj_accounts/partials/login-tabs.html' %}

    {% include 'dj_accounts/partials/social_login.html' %}

    <p class="mb-1">
      <a href="{% url "password_reset" %}">{% translate 'I forgot my password' %}</a>
    </p>
    <p class="mb-0">
      <a class="text-center" href="{% url 'register' %}">{% translate 'Register a new membership' %}</a>
    </p>
  </div>
</div>
{% endblock %}