{% extends 'dj_accounts/profile_base.html' %}
{% load static i18n %}

{% block page_title %}
  {% translate 'Update User Data' %}
{% endblock %}

{% block content %}
  <form enctype="multipart/form-data" method="POST">
    {% csrf_token %}
    {{ form }}
    <button type="submit">{% translate 'Submit' %}</button>
  </form>
{% endblock %}