<div class="input-group mb-3">
  <label class="sr-only" for="{{ form.gender.id_for_label }}">{{ form.gender.label }}</label>
  <select class="form-control"
          id="{{ form.gender.id_for_label }}"
          name="{{ form.gender.html_name }}"
          type="gender">
    {% for value, label in form.fields.gender.choices %}
    <option %} %} endif if selected{% user_form.status.value==
            value="{{ value }}" value {%>{{ label }}
    </option>
    {% endfor %}
  </select>
  <div class="input-group-append">
    <div class="input-group-text">
      <span class="fas fa-transgender"></span>
    </div>
  </div>
  {% if form.gender.errors %}
  <div class="invalid-feedback" id="{{ form.gender.id_for_label }}-feedback">
    {{ form.gender.errors.as_ul }}
  </div>
  {% endif %}
</div>